@@ -19,6 +19,23 @@ soup_address_family_get_type (void)
 	return etype;
 }
 
+/* enumerations from "soup-cookie-jar.h" */
+GType
+soup_cookie_jar_accept_policy_get_type (void)
+{
+	static GType etype = 0;
+	if (G_UNLIKELY (etype == 0)) {
+		static const GEnumValue values[] = {
+			{ SOUP_COOKIE_JAR_ACCEPT_ALWAYS, "SOUP_COOKIE_JAR_ACCEPT_ALWAYS", "always" },
+			{ SOUP_COOKIE_JAR_ACCEPT_NEVER, "SOUP_COOKIE_JAR_ACCEPT_NEVER", "never" },
+			{ SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY, "SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY", "no-third-party" },
+			{ 0, NULL, NULL }
+		};
+		etype = g_enum_register_static (g_intern_static_string ("SoupCookieJarAcceptPolicy"), values);
+	}
+	return etype;
+}
+
 /* enumerations from "soup-date.h" */
 GType
 soup_date_format_get_type (void)
@@ -80,8 +97,10 @@ soup_message_flags_get_type (void)
 	static GType etype = 0;
 	if (G_UNLIKELY (etype == 0)) {
 		static const GFlagsValue values[] = {
-			{ SOUP_MESSAGE_OVERWRITE_CHUNKS, "SOUP_MESSAGE_OVERWRITE_CHUNKS", "overwrite-chunks" },
 			{ SOUP_MESSAGE_NO_REDIRECT, "SOUP_MESSAGE_NO_REDIRECT", "no-redirect" },
+			{ SOUP_MESSAGE_OVERWRITE_CHUNKS, "SOUP_MESSAGE_OVERWRITE_CHUNKS", "overwrite-chunks" },
+			{ SOUP_MESSAGE_CONTENT_DECODED, "SOUP_MESSAGE_CONTENT_DECODED", "content-decoded" },
+			{ SOUP_MESSAGE_CERTIFICATE_TRUSTED, "SOUP_MESSAGE_CERTIFICATE_TRUSTED", "certificate-trusted" },
 			{ 0, NULL, NULL }
 		};
 		etype = g_flags_register_static (g_intern_static_string ("SoupMessageFlags"), values);
@@ -175,6 +194,25 @@ soup_ssl_error_get_type (void)
 	return etype;
 }
 
+GType
+soup_connection_state_get_type (void)
+{
+	static GType etype = 0;
+	if (G_UNLIKELY (etype == 0)) {
+		static const GEnumValue values[] = {
+			{ SOUP_CONNECTION_NEW, "SOUP_CONNECTION_NEW", "new" },
+			{ SOUP_CONNECTION_CONNECTING, "SOUP_CONNECTION_CONNECTING", "connecting" },
+			{ SOUP_CONNECTION_IDLE, "SOUP_CONNECTION_IDLE", "idle" },
+			{ SOUP_CONNECTION_IN_USE, "SOUP_CONNECTION_IN_USE", "in-use" },
+			{ SOUP_CONNECTION_REMOTE_DISCONNECTED, "SOUP_CONNECTION_REMOTE_DISCONNECTED", "remote-disconnected" },
+			{ SOUP_CONNECTION_DISCONNECTED, "SOUP_CONNECTION_DISCONNECTED", "disconnected" },
+			{ 0, NULL, NULL }
+		};
+		etype = g_enum_register_static (g_intern_static_string ("SoupConnectionState"), values);
+	}
+	return etype;
+}
+
 /* enumerations from "soup-socket.h" */
 GType
 soup_socket_io_status_get_type (void)
